@@ -9,7 +9,7 @@ extern char *bas_argv0;
 extern char **bas_argv;
 extern int bas_end;
 
-extern void bas_init(int backslash_colon, int uppercase, int lpfd);
+extern void bas_init(int backslash_colon, int restricted, int uppercase, int lpfd);
 extern void bas_runFile(const char *runFile);
 extern void bas_runLine(const char *runLine);
 extern void bas_interpreter(void);
