@@ -46,6 +46,7 @@ static int used;
 static struct termios origMode,rawMode;
 static const int open_mode[4]={ 0, O_RDONLY, O_WRONLY, O_RDWR };
 static struct sigaction old_sigint, old_sigquit;
+static int termchannel;
 
 const char *FS_errmsg;
 static char FS_errmsgbuf[80];
@@ -144,7 +145,7 @@ static int refill(int dev) /*{{{*/
   }
 }
 /*}}}*/
-static int edit(int chn, int onl) /*{{{*/
+static int edit(int chn, int output_nl) /*{{{*/
 {
   struct FileStream *f=file[chn];
   char *buf=f->inBuf;
@@ -194,18 +195,29 @@ static int edit(int chn, int onl) /*{{{*/
         }
         else FS_putChar(chn,ch);
       }
-      else if (onl) FS_putChar(chn,'\n');
+      else if (output_nl) FS_putChar(chn,'\n');
       f->inBuf[f->inCapacity++]=ch;
     }
   } while (ch!='\n');
   return 0;
 }
 /*}}}*/
+static int outc(int ch) /*{{{*/
+{
+  struct FileStream *f;
+
+  if (opened(termchannel,0)==-1) return -1;
+  f=file[termchannel];
+  if (f->outSize+1>=f->outCapacity && FS_flush(termchannel)==-1) return -1;
+  f->outBuf[f->outSize++]=ch;
+  FS_errmsg=(const char*)0;
+  return ch;
+}
+/*}}}*/
 #ifdef HAVE_TGETENT
 static char *term,entrybuf[2048],*cap;
-static char *cl,*cm,*ce,*md,*me,*AF,*AB;
+static char *cl,*cm,*ce,*cr,*md,*me,*AF,*AB;
 static int Co,NC;
-static int termchannel;
 
 static int mytputs(const char *str, int affcnt, int (*out)(int)) /*{{{*/
 {
@@ -223,6 +235,7 @@ static int initTerminal(int chn) /*{{{*/
 
   if (!init)
   {
+    termchannel=chn;
     if ((term=getenv("TERM"))==(char*)0)
     {
       FS_errmsg=_("environment variable TERM is not set");
@@ -246,6 +259,7 @@ static int initTerminal(int chn) /*{{{*/
         cl=tgetstr("cl",&cap);
         cm=tgetstr("cm",&cap);
         ce=tgetstr("ce",&cap);
+        cr=tgetstr("cr",&cap);
         md=tgetstr("md",&cap);
         me=tgetstr("me",&cap);
         AF=tgetstr("AF",&cap);
@@ -260,22 +274,8 @@ static int initTerminal(int chn) /*{{{*/
   return 0;
 }
 /*}}}*/
-static int outc(int ch) /*{{{*/
-{
-  struct FileStream *f;
-
-  if (opened(termchannel,0)==-1) return -1;
-  f=file[termchannel];
-  if (f->outSize+1>=f->outCapacity && FS_flush(termchannel)==-1) return -1;
-  f->outBuf[f->outSize++]=ch;
-  FS_errmsg=(const char*)0;
-  return ch;
-}
-/*}}}*/
 static int cls(int chn) /*{{{*/
 {
-  if (initTerminal(chn)==-1) return -1;
-  termchannel=chn;
   if (cl==(char*)0)
   {
     sprintf(FS_errmsgbuf,_("terminal type %s can not clear the screen"),term);
@@ -288,7 +288,6 @@ static int cls(int chn) /*{{{*/
 /*}}}*/
 static int locate(int chn, int line, int column) /*{{{*/
 {
-  if (initTerminal(chn)==-1) return -1;
   termchannel=chn;
   if (cm==(char*)0)
   {
@@ -302,8 +301,6 @@ static int locate(int chn, int line, int column) /*{{{*/
 /*}}}*/
 static int colour(int chn, int foreground, int background) /*{{{*/
 {
-  if (initTerminal(chn)==-1) return -1;
-  termchannel=chn;
   if (AF && AB && Co>=8)
   {
     static int map[8]={ 0,4,2,6,1,5,3,7 };
@@ -340,7 +337,20 @@ static int resetcolour(int chn) /*{{{*/
   return 0;
 }
 /*}}}*/
+static void crlf(int chn) /*{{{*/
+{
+  if (cr) mytputs(cr,0,outc);
+  else outc('\r');
+  outc('\n');
+}
+/*}}}*/
 #else
+static int initTerminal(int chn) /*{{{*/
+{
+  termchannel=chn;
+  return 0;
+}
+/*}}}*/
 static int cls(int chn) /*{{{*/
 {
   FS_errmsg=_("This installation does not support terminal handling");
@@ -364,6 +374,12 @@ static int resetcolour(int chn) /*{{{*/
   return 0;
 }
 /*}}}*/
+static void crlf(int chn) /*{{{*/
+{
+  outc('\r');
+  outc('\n');
+}
+/*}}}*/
 #endif
 static void sigintr(int sig) /*{{{*/
 {
@@ -382,7 +398,7 @@ int FS_opendev(int chn, int infd, int outfd) /*{{{*/
   }
   file[chn]=malloc(sizeof(struct FileStream));
   file[chn]->dev=1;
-  if ((file[chn]->tty=(infd==0 ? isatty(infd) : 0)))
+  if ((file[chn]->tty=(infd==0 ? isatty(infd) && isatty(outfd) : 0)))
   {
     if (tcgetattr(infd,&origMode)==-1)
     {
@@ -395,6 +411,7 @@ int FS_opendev(int chn, int infd, int outfd) /*{{{*/
     rawMode.c_lflag&=~(ICANON|ECHO); /* IEXTEN would disable IUCLC, breaking UC only terminals */
     rawMode.c_cc[VMIN]=1;
     rawMode.c_cc[VTIME]=0;
+    rawMode.c_oflag&=~ONLCR;
     if (tcsetattr(infd,TCSADRAIN,&rawMode)==-1)
     {
       FS_errmsg=strerror(errno);
@@ -402,6 +419,7 @@ int FS_opendev(int chn, int infd, int outfd) /*{{{*/
       file[chn]=(struct FileStream*)0;
       return -1;
     }
+    initTerminal(chn);
   }
   file[chn]->recLength=1;
   file[chn]->infd=infd;
@@ -754,7 +772,7 @@ int FS_truncate(int chn) /*{{{*/
   if ((fd=file[chn]->outfd)==-1)
   if ((fd=file[chn]->randomfd)==-1)
   if ((fd=file[chn]->binaryfd)==-1) assert(0);
-  if ((o=lseek(fd,SEEK_CUR,0))==(off_t)-1 || ftruncate(fd,o+1)==-1)
+  if ((o=lseek(fd,0,SEEK_CUR))==(off_t)-1 || ftruncate(fd,o+1)==-1)
   {
     FS_errmsg=strerror(errno);
     return -1;
@@ -822,10 +840,12 @@ int FS_putChar(int dev, char ch) /*{{{*/
   if (f->outSize+2>=f->outCapacity && FS_flush(dev)==-1) return -1;
   if (f->outLineWidth && f->outPos==f->outLineWidth)
   {
-    f->outBuf[f->outSize++]='\n';
+    if (FS_istty(dev)) crlf(dev);
+    else f->outBuf[f->outSize++]='\n';
     f->outPos=0;
   }
-  f->outBuf[f->outSize++]=ch;
+  if (FS_istty(dev) && ch=='\n') crlf(dev);
+  else f->outBuf[f->outSize++]=ch;
   if (ch!='\n' && ch!='\b') ++f->outPos;
   FS_errmsg=(const char*)0;
   return 0;
@@ -846,6 +866,16 @@ int FS_putString(int dev, const struct String *s) /*{{{*/
   return 0;
 }
 /*}}}*/
+int FS_putItem(int dev, const struct String *s) /*{{{*/
+{
+  struct FileStream *f;
+
+  if (opened(dev,0)==-1) return -1;
+  f=file[dev];
+  if (f->outPos && f->outLineWidth && f->outPos+s->length>f->outLineWidth) FS_nextline(dev);
+  return FS_putString(dev, s);
+}
+/*}}}*/
 int FS_putbinaryString(int chn, const struct String *s) /*{{{*/
 {
   if (opened(chn,3)==-1) return -1;
@@ -940,8 +970,11 @@ int FS_nextcol(int dev) /*{{{*/
     f->outPos%f->outColWidth
     && f->outLineWidth
     && ((f->outPos/f->outColWidth+2)*f->outColWidth)>f->outLineWidth
-    && FS_putChar(dev,'\n')==-1
-  ) return -1;
+  )
+  {
+    return FS_putChar(dev,'\n');
+  }
+  if (!(f->outPos%f->outColWidth) && FS_putChar(dev,' ')==-1) return -1;
   while (f->outPos%f->outColWidth) if (FS_putChar(dev,' ')==-1) return -1;
   return 0;
 }
@@ -1234,7 +1267,7 @@ int FS_seek(int chn, long int record) /*{{{*/
   return -1;
 }
 /*}}}*/
-int FS_appendToString(int chn, struct String *s, int onl) /*{{{*/
+int FS_appendToString(int chn, struct String *s, int output_nl) /*{{{*/
 {
   size_t new;
   char *n;
@@ -1243,7 +1276,7 @@ int FS_appendToString(int chn, struct String *s, int onl) /*{{{*/
 
   if (f->tty && f->inSize==f->inCapacity)
   {
-    if (edit(chn,onl)==-1) return (FS_errmsg ? -1 : 0);
+    if (edit(chn,output_nl)==-1) return (FS_errmsg ? -1 : 0);
   }
   do
   {
