@@ -406,6 +406,16 @@ static struct Value *fn_commandd(struct Value *v, struct Auto *stack) /*{{{*/
   return v;
 }
 /*}}}*/
+static struct Value *fn_cvd(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  struct String *s=stringValue(stack,0);
+  double n;
+
+  if (s->length!=sizeof(double)) return Value_new_ERROR(v,BADCONVERSION,_("number"));
+  memcpy(&n,s->character,sizeof(double));
+  return Value_new_REAL(v,n);
+}
+/*}}}*/
 static struct Value *fn_cvi(struct Value *v, struct Auto *stack) /*{{{*/
 {
   struct String *s=stringValue(stack,0);
@@ -426,16 +436,6 @@ static struct Value *fn_cvs(struct Value *v, struct Auto *stack) /*{{{*/
   return Value_new_REAL(v,(double)n);
 }
 /*}}}*/
-static struct Value *fn_cvd(struct Value *v, struct Auto *stack) /*{{{*/
-{
-  struct String *s=stringValue(stack,0);
-  double n;
-
-  if (s->length!=sizeof(double)) return Value_new_ERROR(v,BADCONVERSION,_("number"));
-  memcpy(&n,s->character,sizeof(double));
-  return Value_new_REAL(v,n);
-}
-/*}}}*/
 static struct Value *fn_date(struct Value *v, struct Auto *stack) /*{{{*/
 {
   time_t t;
@@ -621,6 +621,12 @@ static struct Value *fn_fix(struct Value *v, struct Auto *stack) /*{{{*/
   return Value_new_REAL(v,x<0.0?ceil(x):floor(x));
 }
 /*}}}*/
+static struct Value *fn_frac(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  double x=realValue(stack,0);
+  return Value_new_REAL(v,x<0.0 ? x-ceil(x) : x-floor(x));
+}
+/*}}}*/
 static struct Value *fn_freefile(struct Value *v, struct Auto *stack) /*{{{*/
 {
   return Value_new_INTEGER(v,FS_freechn());
@@ -907,6 +913,20 @@ static struct Value *fn_log(struct Value *v, struct Auto *stack) /*{{{*/
   return v;
 }
 /*}}}*/
+static struct Value *fn_log10(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  if (realValue(stack,0)<=0.0) Value_new_ERROR(v,UNDEFINED,_("Logarithm of negative value"));
+  else Value_new_REAL(v,log10(realValue(stack,0)));
+  return v;
+}
+/*}}}*/
+static struct Value *fn_log2(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  if (realValue(stack,0)<=0.0) Value_new_ERROR(v,UNDEFINED,_("Logarithm of negative value"));
+  else Value_new_REAL(v,log2(realValue(stack,0)));
+  return v;
+}
+/*}}}*/
 static struct Value *fn_ltrim(struct Value *v, struct Auto *stack) /*{{{*/
 {
   struct String *s=stringValue(stack,0);
@@ -961,6 +981,44 @@ static struct Value *fn_match(struct Value *v, struct Auto *stack) /*{{{*/
   return Value_new_INTEGER(v,0);
 }
 /*}}}*/
+static struct Value *fn_maxii(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  long int x,y;
+
+  x=intValue(stack,0);
+  y=intValue(stack,1);
+  return Value_new_INTEGER(v,x>y?x:y);
+}
+/*}}}*/
+static struct Value *fn_maxdi(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  double x;
+  long int y;
+
+  x=realValue(stack,0);
+  y=intValue(stack,1);
+  return Value_new_REAL(v,x>y?x:y);
+}
+/*}}}*/
+static struct Value *fn_maxid(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  long int x;
+  double y;
+
+  x=intValue(stack,0);
+  y=realValue(stack,1);
+  return Value_new_REAL(v,x>y?x:y);
+}
+/*}}}*/
+static struct Value *fn_maxdd(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  double x,y;
+
+  x=realValue(stack,0);
+  y=realValue(stack,1);
+  return Value_new_REAL(v,x>y?x:y);
+}
+/*}}}*/
 static struct Value *fn_mid2i(struct Value *v, struct Auto *stack) /*{{{*/
 {
   return mid(v,stringValue(stack,0),intValue(stack,1),stringValue(stack,0)->length);
@@ -1014,6 +1072,44 @@ static struct Value *fn_mid3dd(struct Value *v, struct Auto *stack) /*{{{*/
   return mid(v,stringValue(stack,0),start,len);
 }
 /*}}}*/
+static struct Value *fn_minii(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  long int x,y;
+
+  x=intValue(stack,0);
+  y=intValue(stack,1);
+  return Value_new_INTEGER(v,x<y?x:y);
+}
+/*}}}*/
+static struct Value *fn_mindi(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  double x;
+  long int y;
+
+  x=realValue(stack,0);
+  y=intValue(stack,1);
+  return Value_new_REAL(v,x<y?x:y);
+}
+/*}}}*/
+static struct Value *fn_minid(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  long int x;
+  double y;
+
+  x=intValue(stack,0);
+  y=realValue(stack,1);
+  return Value_new_REAL(v,x<y?x:y);
+}
+/*}}}*/
+static struct Value *fn_mindd(struct Value *v, struct Auto *stack) /*{{{*/
+{
+  double x,y;
+
+  x=realValue(stack,0);
+  y=realValue(stack,1);
+  return Value_new_REAL(v,x<y?x:y);
+}
+/*}}}*/
 static struct Value *fn_mki(struct Value *v, struct Auto *stack) /*{{{*/
 {
   long int x=intValue(stack,0);
@@ -1379,9 +1475,9 @@ struct Global *Global_new(struct Global *this) /*{{{*/
   builtin(this,"command$",V_STRING, fn_commandi,  1,V_INTEGER);
   builtin(this,"command$",V_STRING, fn_commandd,  1,V_REAL);
   builtin(this,"cos",     V_REAL,   fn_cos,       1,V_REAL);
+  builtin(this,"cvd",     V_REAL,   fn_cvd,       1,V_STRING);
   builtin(this,"cvi",     V_INTEGER,fn_cvi,       1,V_STRING);
   builtin(this,"cvs",     V_REAL,   fn_cvs,       1,V_STRING);
-  builtin(this,"cvd",     V_REAL,   fn_cvd,       1,V_STRING);
   builtin(this,"date$",   V_STRING, fn_date,      0);
   builtin(this,"dec$",    V_STRING, fn_dec,       2,V_REAL,V_STRING);
   builtin(this,"dec$",    V_STRING, fn_dec,       2,V_INTEGER,V_STRING);
@@ -1401,7 +1497,9 @@ struct Global *Global_new(struct Global *this) /*{{{*/
   builtin(this,"find$",   V_STRING, fn_findi,     2,V_STRING,V_INTEGER);
   builtin(this,"find$",   V_STRING, fn_findd,     2,V_STRING,V_REAL);
   builtin(this,"fix",     V_REAL,   fn_fix,       1,V_REAL);
+  builtin(this,"frac",    V_REAL,   fn_frac,      1,V_REAL);
   builtin(this,"freefile",V_INTEGER,fn_freefile,  0);
+  builtin(this,"fp",      V_REAL,   fn_frac,      1,V_REAL);
   builtin(this,"hex$",    V_STRING, fn_hexi,      1,V_INTEGER);
   builtin(this,"hex$",    V_STRING, fn_hexd,      1,V_REAL);
   builtin(this,"hex$",    V_STRING, fn_hexii,     2,V_INTEGER,V_INTEGER);
@@ -1429,6 +1527,7 @@ struct Global *Global_new(struct Global *this) /*{{{*/
   builtin(this,"instr",   V_INTEGER,fn_instr4dd,  4,V_STRING,V_STRING,V_REAL,V_REAL);
   builtin(this,"int",     V_REAL,   fn_int,       1,V_REAL);
   builtin(this,"int%",    V_INTEGER,fn_intp,      1,V_REAL);
+  builtin(this,"ip",      V_REAL,   fn_fix,       1,V_REAL);
   builtin(this,"lcase$",  V_STRING, fn_lcase,     1,V_STRING);
   builtin(this,"lower$",  V_STRING, fn_lcase,     1,V_STRING);
   builtin(this,"left$",   V_STRING, fn_left,      2,V_STRING,V_INTEGER);
@@ -1436,14 +1535,24 @@ struct Global *Global_new(struct Global *this) /*{{{*/
   builtin(this,"loc",     V_INTEGER,fn_loc,       1,V_INTEGER);
   builtin(this,"lof",     V_INTEGER,fn_lof,       1,V_INTEGER);
   builtin(this,"log",     V_REAL,   fn_log,       1,V_REAL);
+  builtin(this,"log10",   V_REAL,   fn_log10,     1,V_REAL);
+  builtin(this,"log2",    V_REAL,   fn_log2,      1,V_REAL);
   builtin(this,"ltrim$",  V_STRING, fn_ltrim,     1,V_STRING);
   builtin(this,"match",   V_INTEGER,fn_match,     3,V_STRING,V_STRING,V_INTEGER);
+  builtin(this,"max",     V_INTEGER,fn_maxii,     2,V_INTEGER,V_INTEGER);
+  builtin(this,"max",     V_REAL,   fn_maxdi,     2,V_REAL,V_INTEGER);
+  builtin(this,"max",     V_REAL,   fn_maxid,     2,V_INTEGER,V_REAL);
+  builtin(this,"max",     V_REAL,   fn_maxdd,     2,V_REAL,V_REAL);
   builtin(this,"mid$",    V_STRING, fn_mid2i,     2,V_STRING,V_INTEGER);
   builtin(this,"mid$",    V_STRING, fn_mid2d,     2,V_STRING,V_REAL);
   builtin(this,"mid$",    V_STRING, fn_mid3ii,    3,V_STRING,V_INTEGER,V_INTEGER);
   builtin(this,"mid$",    V_STRING, fn_mid3id,    3,V_STRING,V_INTEGER,V_REAL);
   builtin(this,"mid$",    V_STRING, fn_mid3di,    3,V_STRING,V_REAL,V_INTEGER);
   builtin(this,"mid$",    V_STRING, fn_mid3dd,    3,V_STRING,V_REAL,V_REAL);
+  builtin(this,"min",     V_INTEGER,fn_minii,     2,V_INTEGER,V_INTEGER);
+  builtin(this,"min",     V_REAL,   fn_mindi,     2,V_REAL,V_INTEGER);
+  builtin(this,"min",     V_REAL,   fn_minid,     2,V_INTEGER,V_REAL);
+  builtin(this,"min",     V_REAL,   fn_mindd,     2,V_REAL,V_REAL);
   builtin(this,"mki$",    V_STRING, fn_mki,       1,V_INTEGER);
   builtin(this,"mks$",    V_STRING, fn_mks,       1,V_REAL);
   builtin(this,"mkd$",    V_STRING, fn_mkd,       1,V_REAL);
